# Defined type wrapper for firmware
#
# $product = product name that you will be updating (see case statement for hint)
# $version = intended version number that the firmware should be on
# $path = path to the fiware binary OR the catalog.xml file for the idrac
# $force_restart = automatically restart the iDRAC server if reboots are needed (don't wait on user to reboot)
# $asm_hostname = the IP address of the appliance to get the firmware binaries from
#

define asm::firmware (
  $install_type    = "uri",
  $force_restart   = false,
  $decrypt         = false,
  $product         = undef,
  $version         = undef,
  $path            = undef,
  $asm_hostname    = undef,
  $esx_hostname    = undef,
  $esx_password    = undef,
  $server_firmware = undef,
  $instance_id     = undef,
  $vcenter_cert    = undef
  ) {

  $tftp_share = '/var/lib/tftpboot'

  $fw_type = $product ? {
    /^cmc.*/                   => 'cmc',
    /^dell_iom.*|^dell_ftos.*/ => 'ftos',
    /^idrac.*/                 => 'dell_server',
    /^server.*/                => 'bmc',
    /^dell_powerconnect.*/     => 'powerconnect',
    default                    => 'other'
  }

  case $fw_type {
    'cmc': {
      if $path =~ /\/var\/nfs\/firmware\/(.*)/ {
        $tftp_path = $1
      }
      cmc_fw_update {"update":
        ensure         => present,
        version        => $version,
        path           => $path,
        asm_hostname   => $asm_hostname,
        copy_to_tftp   => [$tftp_share,$tftp_path]
      }
    }
    'bmc': {
      if $path =~ /\/var\/nfs\/firmware\/(.*)\// {
        $firmware_repo_id = $1
      }
      bmc_fw_ipmiflash { "update":
        bmc_firmware => exe_to_bmc_fw($server_firmware, $asm_hostname, $tftp_share, $firmware_repo_id),
        ensure       => present,
      }
    }
    'ftos': {
      if $path =~ /\/var\/nfs\/firmware\/(.*)/ {
        $tftp_path = $1
      }
      force10_firmwareupdate {"firmware_update":
        force        => false,
        url          => "tftp://$asm_hostname/$tftp_path",
        copy_to_tftp => [$tftp_share,$tftp_path],
        path         => $path,
      }
    }
    'powerconnect': {
      if $path =~ /\/var\/nfs\/firmware\/(.*)/ {
        $tftp_path = $1
      }
      $tftp_url = "tftp://$asm_hostname/$tftp_path"
      $imageurl = nfs_to_tftp($path,$tftp_url,$tftp_share)
      powerconnect_firmware { "firmware":
        imageurl    => $imageurl,
        forceupdate => 'true',
        saveconfig  => 'true'
      }
    }
    'dell_server': {
#  If esx_hostname is defined. put the host in maintenance mode before update and take out of maintenance
#  mode on successful update
      if ($esx_hostname != undef) and ($force_restart == true) {
        if ($vcenter_cert != undef) {
          transport { 'vcenter' :
            name     => $vcenter_cert,
            options  => {'insecure' => true},
            provider => 'device_file'
          }
        }
        else {
          transport { 'vcenter' :
            username => 'root',
            password => asm_cipher_decrypt($esx_password),
            server   => $esx_hostname,
            options  => { 'insecure' => true },
          }
        }
        esx_maintmode { 'enable' :
          ensure                   => present,
          timeout                  => 300,
          transport                => Transport['vcenter'],
          evacuate_powered_off_vms => true,
          host                     => $esx_hostname,
        }
        if $path =~ /^(.+)\/([^\/]+)$/ {
          $share_path = $1
        }
        file_line { "create_share":
          path   => '/etc/exports',
          line   => "$share_path     *(ro,sync,no_subtree_check)",
          notify => Service["nfs"],
        }
        service { "nfs":
          ensure => running,
        }
        if $install_type == 'uri' {
          idrac_fw_installfromuri {$product :
            ensure         => present,
            force_restart  => $force_restart,
            idrac_firmware => path_to_uri($server_firmware, $asm_hostname),
            require        => [Esx_maintmode['enable'],File_line['create_share']],
            before         => Esx_maintmode['disable']
          }
          Idrac_fw_installfromuri[$product] -> Exec['wait_for_esxi_services'] -> Esx_maintmode['disable']
        }
        else {
          idrac_fw_update { "update":
            name          => $name,
            asm_hostname  => $asm_hostname,
            path          => $path,
            force_restart => $force_restart,
            require       => [Esx_maintmode['enable'],File_line['create_share']],
            before        => Esx_maintmode['disable']
          }
          Idrac_fw_update["update"] -> Exec['wait_for_esxi_services'] -> Esx_maintmode['disable']
        }
        exec { 'wait_for_esxi_services' :
          command => '/bin/sleep 120',
        }
        if ($vcenter_cert != undef) {
          esx_maintmode { 'disable':
            ensure                   => absent,
            timeout                  => 300,
            transport                => Transport['vcenter'],
            host                     => $esx_hostname,
            evacuate_powered_off_vms => true,
          }
        }
        else {
          transport { 'disableVcenter' :
            username => 'root',
            password => asm_cipher_decrypt($esx_password),
            server   => $esx_hostname,
            options  => { 'insecure' => true },
          }
          esx_maintmode { 'disable':
            ensure                   => absent,
            timeout                  => 300,
            transport                => Transport['disableVcenter'],
            host                     => $esx_hostname,
            evacuate_powered_off_vms => true,
          }
        }
      }
      else  {
        if $path =~ /^(.+)\/([^\/]+)$/ {
          $share_path = $1
        }
        file_line { "create_share":
          path   => '/etc/exports',
          line   => "$share_path     *(ro,sync,no_subtree_check)",
          notify => Service["nfs"],
        }
        service { "nfs":
          ensure => running,
        }
        if $install_type == 'uri' {
          idrac_fw_installfromuri {$product :
            ensure         => present,
            force_restart  => $force_restart,
            idrac_firmware => path_to_uri($server_firmware, $asm_hostname),
            require       => File_line['create_share'],
          }
        }
        else {
          idrac_fw_update { "update":
            name          => $name,
            asm_hostname  => $asm_hostname,
            path          => $path,
            force_restart => $force_restart,
            require       => File_line['create_share'],
          }
        }
      }
    }
    default : {
      fail("Firmware for product type: $fw_type not support")
    }
  }

}
